#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 8/12/23

# Inspire by a Photoshop tutorial found here: https://www.youtube.com/watch?v=K81ZiI_yQDs

# Comments directed to http://gimpchat.com

# Copyright (C)

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html


from gimpfu import *
import sys,os,os
import string
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_shadow_and_reflection_text(customtext, fontname):

    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    #Create text as a guide an later delete it
    size = 250
    img = pdb.gimp_image_new(1, 1, RGB)
    pdb.gimp_context_set_foreground("#000000")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 100, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, -7)
    pdb.gimp_text_layer_set_line_spacing(textLayer, -54)
    
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
    
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
   
    #Image process
    gimp.Display(img)  
    
    #Get text and create a layer group
    fontname = pdb.gimp_text_layer_get_font(textLayer)
    size,unit = pdb.gimp_text_layer_get_font_size(textLayer)
    new_layer_group = pdb.gimp_layer_group_new(img)
    pdb.gimp_image_insert_layer(img, new_layer_group, None, 0)
       
########################### From the plugin MareroQ "Random wobble text layer.py" start ######################################################
    
    #Separate the text in individual layers and align text
    offset_x,offset_y = pdb.gimp_drawable_offsets(textLayer)
    lines = customtext.split("\n")
    starting_y = offset_y
    for line in lines:
        textLayers = []
        totalwidth = 0
        for i in range(0,len(line)):
            char = line[i]
            if ((char == " ") and (i > 0) and (i < len(line)-1)): #if space and it's between 2 chararacters we treat it special
                before = line[i-1]
                space = line[i]
                after = line[i+1]
                special = before + space + after
                special_layer = pdb.gimp_text_fontname(img, None, 0, 0, special, 0, TRUE, size, unit, fontname)
                before_layer = pdb.gimp_text_fontname(img, None, 0, 0,before ,0 ,TRUE ,size ,unit, fontname)
                after_layer = pdb.gimp_text_fontname(img, None, 0, 0, after, 0, TRUE, size, unit, fontname)
                layer_height = special_layer.height
                layer_width = special_layer.width - before_layer.width - after_layer.width
                textLayer = pdb.gimp_layer_new(img, layer_width, layer_height, RGBA_IMAGE, "Space", 100, LAYER_MODE_NORMAL)
                pdb.gimp_image_insert_layer(img, textLayer, new_layer_group, 0)
                pdb.gimp_image_remove_layer(img, before_layer)
                pdb.gimp_image_remove_layer(img, after_layer)
                pdb.gimp_image_remove_layer(img, special_layer)
            else:
                textLayer = pdb.gimp_text_fontname(img, None, 0, 0, char, 0, TRUE, size, unit, fontname)
                pdb.gimp_image_reorder_item(img, textLayer, new_layer_group, 0) #puts it inside our layer group for organization
            totalwidth += textLayer.width #increment total width to calculate for placement and justify.
            textLayers.append(textLayer)
        
        starting_x = offset_x
        justify = []
        if (justify == 0): #left justify
            starting_x = offset_x
        elif (justify == 1): #centered
            starting_x = offset_x + (textLayer.width/2) - totalwidth/2
        elif (justify == 2): #right justify
            starting_x = offset_x + textLayer.width - totalwidth
        for textLayer in textLayers: #loop through and move it into its placement
            pdb.gimp_layer_set_offsets(textLayer, starting_x, starting_y)
            starting_x += textLayer.width
        starting_y += textLayer.height #increment it for next round/line
     
########################### MareroQ "Random wobble text layer.py" end ######################################################

    #Delete original layer
    text = pdb.gimp_image_get_layer_by_name(img, "Text")
    pdb.gimp_image_remove_layer(img, text)
    
    #Center text group
    x = new_layer_group.offsets[0]
    y = new_layer_group.offsets[1]
    x = (img.width - new_layer_group.width) / 2
    y = (img.height - new_layer_group.height) / 2
    new_layer_group.set_offsets(x, y)
    
    #Ungroup layers 
    pdb.gimp_item_is_group(new_layer_group)
    parent = pdb.gimp_item_get_parent(new_layer_group)
    position = pdb.gimp_image_get_item_position(img, new_layer_group)
    for layer in new_layer_group.layers[::-1]:
        pdb.gimp_image_reorder_item(img, layer, parent, position)
        
    #Delete group folder
    pdb.gimp_image_remove_layer(img, new_layer_group)
    
    #Rename the layers
    applyvisible = 0
    prefix = 0
    applylayers = []
    counter = 0
    for layer in img.layers:
        counter += 1
        if (applyvisible == 1) or (layer.visible != 0):
            applylayers.append([layer,counter])                
    for applylayer in applylayers:
        applylayer[0].name = "Layer #1" 
    
    #Reverse visible layers
    layerlist = img.layers
    processlist=[]
    emptycheck =[]
    
    for i in range(len(layerlist)):
        visible = pdb.gimp_item_get_visible(layerlist[i])
        if visible:
            processlist.append(layerlist[i])
    count = 0
    for i in processlist:
        pdb.gimp_image_raise_item_to_top(img,i)
        count+=1
    
    #Create the background 
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, backLayer, None, 0)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND) 
    pdb.gimp_image_lower_item_to_bottom(img, backLayer)
    pdb.gimp_item_set_visible(backLayer, FALSE)
            
    #Create shadows 
    wh1 = 0
    hh1 = layer.height /2
    wh2 = layer.width + 40
    hh2 = layer.height /2
    pdb.gimp_context_set_default_colors()
    layerlist = img.layers
    processlist=[]
    emptycheck =[]
    for i in range(len(layerlist)):
        visible = pdb.gimp_item_get_visible(layerlist[i])
        if visible:
            processlist.append(layerlist[i])
    count = 0
    for i in processlist:
        pdb.gimp_image_select_item(img, 2, i)   
        pdb.gimp_context_set_gradient_fg_bg_rgb()
        pdb.gimp_context_set_gradient_reverse(TRUE)
        pdb.gimp_context_set_gradient_blend_color_space(GRADIENT_BLEND_RGB_PERCEPTUAL)
        pdb.gimp_drawable_edit_gradient_fill(i, 
                                             0,   #gradient type - linear
                                             0,   #offset
                                         FALSE,   #supersample
                                             3,   #max-depth
                                          0.20,   #threshold
                                          TRUE,   #dither
                                           wh1,   #width1
                                           hh1,   #height1
                                           wh2,   #width2
                                           hh2)   #height2   
        pdb.gimp_selection_none(img)
        count+=1
    
    #Make invisible top and bottom layers
    topLayer = img.layers[0]
    pdb.gimp_item_set_visible(topLayer, FALSE)
    pdb.gimp_item_set_visible(backLayer, FALSE)
    
    #Link all visible layers
    layerlist = img.layers
    processlist=[]
    emptycheck =[]
    for i in range(len(layerlist)):
        visible = pdb.gimp_item_get_visible(layerlist[i])
        if visible:
            processlist.append(layerlist[i])
    count = 0
    for i in processlist:
        pdb.gimp_item_set_linked(i,TRUE)
        count+=1
    
    #Make visible top layer
    pdb.gimp_item_set_visible(topLayer, TRUE)
        
    #Drag layers closer
    layerlist = img.layers
    processlist=[]
    emptycheck =[]
    count = 0
    for i in range(len(layerlist)):
        top = pdb.gimp_item_get_linked(layerlist[i])
        if top:
            processlist.append(layerlist[i])
    for i in processlist: 
        pdb.gimp_item_transform_translate(i, -35, 0)
        pdb.gimp_item_set_linked(i,FALSE)
        count+=1 
    
    #Reverse visible layers again
    layerlist = img.layers
    processlist=[]
    emptycheck =[]
    
    for i in range(len(layerlist)):
        visible = pdb.gimp_item_get_visible(layerlist[i])
        if visible:
            processlist.append(layerlist[i])
    count = 0
    for i in processlist:
        pdb.gimp_image_raise_item_to_top(img,i)
        count+=1
    
    #Merge all layer into one
    textLayer = pdb.gimp_image_merge_visible_layers(img, EXPAND_AS_NECESSARY)
    
    #Rename the text layer and center the text
    topLayer = img.layers[0]
    pdb.gimp_item_set_name(topLayer, "Text")
    x = topLayer.offsets[0]
    y = topLayer.offsets[1]
    x = (img.width - topLayer.width) / 2
    topLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(topLayer)
    
    #Make the background visible
    pdb.gimp_item_set_visible(backLayer, TRUE)
    
    #Created a reflection
    wh1 = img.width /2 
    hh1 = img.height /2 
    wh2 = img.width /2
    hh2 = img.height 
    RefLayer = pdb.gimp_layer_new_from_drawable(topLayer, img)
    pdb.gimp_image_insert_layer(img, RefLayer, None, 1)
    pdb.gimp_item_set_name(RefLayer, "Reflection")
    pdb.gimp_layer_set_mode(RefLayer, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_layer_set_opacity(RefLayer, 65)
    RefLayer = pdb.gimp_drawable_transform_flip_simple(RefLayer, 
                                           ORIENTATION_VERTICAL,   #flip_type 
                                                           TRUE,   #auto_center 
                                                              0,   #axis 
                                                           TRUE)   #clip_result
    
    pdb.gimp_drawable_offset(RefLayer, FALSE, OFFSET_TRANSPARENT, 0, 185)
    mask = pdb.gimp_layer_create_mask(RefLayer, ADD_MASK_BLACK)
    pdb.gimp_layer_add_mask(RefLayer, mask)
    pdb.gimp_context_set_default_colors()
    pdb.gimp_edit_blend(mask, 
                           0,  #blend-mode - blend_fg_bg_rgb
                           0,  #paint-mode - layer_mode_normal
                           0,  #gradient type - linear
                         100,  #opacity
                           0,  #offset
                           0,  #repeat
                        TRUE,  #reverse
                       FALSE,  #supersample
                           3,  #max-depth
                        0.20,  #threshold
                        TRUE,  #dither
                         wh1,  #width1
                         hh1,  #height1
                         wh2,  #width2
                         hh2)  #height2   

    #Set the defaults
    pdb.gimp_context_pop()
register(
    "pm_shadow_and_reflection_text",
    "Creates Shadow text effect",
    "Creates a shadow & Reflection text effect",
    "Pocholo",
    "Pocholo",
    "2023",
    "Shadow Text effect",
    "",
    [
    (PF_STRING, "customtext", "Text string", "SHADOWS"),
    (PF_FONT, "fontname", "Font face", "Franklin Gothic Demi"),
    ],
    [],
    pm_shadow_and_reflection_text, menu="<Image>/Python-Fu/Text Effects",
    domain=("gimp20", gimp.locale_directory))
main()								